@@ -20,11 +20,14 @@
  * Author: Alexander Larsson <alexl@redhat.com>
  */
 
-#include <config.h>
+#include "config.h"
 #include "gfileenumerator.h"
+#include "gfile.h"
 #include "gioscheduler.h"
+#include "gasyncresult.h"
 #include "gasynchelper.h"
 #include "gsimpleasyncresult.h"
+#include "gioerror.h"
 #include "glibintl.h"
 
 #include "gioalias.h"
@@ -56,12 +59,18 @@ G_DEFINE_TYPE (GFileEnumerator, g_file_enumerator, G_TYPE_OBJECT);
 
 struct _GFileEnumeratorPrivate {
   /* TODO: Should be public for subclasses? */
+  GFile *container;
   guint closed : 1;
   guint pending : 1;
   GAsyncReadyCallback outstanding_callback;
   GError *outstanding_error;
 };
 
+enum {
+  PROP_0,
+  PROP_CONTAINER
+};
+
 static void     g_file_enumerator_real_next_files_async  (GFileEnumerator      *enumerator,
 							  int                   num_files,
 							  int                   io_priority,
@@ -80,6 +89,41 @@ static gboolean g_file_enumerator_real_close_finish      (GFileEnumerator      *
 							  GAsyncResult         *res,
 							  GError              **error);
 
+static void
+g_file_enumerator_set_property (GObject      *object,
+                                guint         property_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
+{
+  GFileEnumerator *enumerator;
+  
+  enumerator = G_FILE_ENUMERATOR (object);
+  
+  switch (property_id) {
+  case PROP_CONTAINER:
+    enumerator->priv->container = g_value_dup_object (value);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
+  }
+}
+
+static void
+g_file_enumerator_dispose (GObject *object)
+{
+  GFileEnumerator *enumerator;
+
+  enumerator = G_FILE_ENUMERATOR (object);
+  
+  if (enumerator->priv->container) {
+    g_object_unref (enumerator->priv->container);
+    enumerator->priv->container = NULL;
+  }
+
+  G_OBJECT_CLASS (g_file_enumerator_parent_class)->dispose (object);
+}
+
 static void
 g_file_enumerator_finalize (GObject *object)
 {
@@ -90,8 +134,7 @@ g_file_enumerator_finalize (GObject *object)
   if (!enumerator->priv->closed)
     g_file_enumerator_close (enumerator, NULL, NULL);
 
-  if (G_OBJECT_CLASS (g_file_enumerator_parent_class)->finalize)
-    (*G_OBJECT_CLASS (g_file_enumerator_parent_class)->finalize) (object);
+  G_OBJECT_CLASS (g_file_enumerator_parent_class)->finalize (object);
 }
 
 static void
@@ -101,12 +144,23 @@ g_file_enumerator_class_init (GFileEnumeratorClass *klass)
   
   g_type_class_add_private (klass, sizeof (GFileEnumeratorPrivate));
   
+  gobject_class->set_property = g_file_enumerator_set_property;
+  gobject_class->dispose = g_file_enumerator_dispose;
   gobject_class->finalize = g_file_enumerator_finalize;
 
   klass->next_files_async = g_file_enumerator_real_next_files_async;
   klass->next_files_finish = g_file_enumerator_real_next_files_finish;
   klass->close_async = g_file_enumerator_real_close_async;
   klass->close_finish = g_file_enumerator_real_close_finish;
+
+  g_object_class_install_property
+    (gobject_class, PROP_CONTAINER,
+     g_param_spec_object ("container", P_("Container"),
+                          P_("The container that is being enumerated"),
+                          G_TYPE_FILE,
+                          G_PARAM_WRITABLE |
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -147,15 +201,15 @@ g_file_enumerator_next_file (GFileEnumerator *enumerator,
   
   if (enumerator->priv->closed)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_CLOSED,
-		   _("Enumerator is closed"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_CLOSED,
+                           _("Enumerator is closed"));
       return NULL;
     }
 
   if (enumerator->priv->pending)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_PENDING,
-		   _("File enumerator has outstanding operation"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_PENDING,
+                           _("File enumerator has outstanding operation"));
       return NULL;
     }
 
@@ -213,8 +267,8 @@ g_file_enumerator_close (GFileEnumerator  *enumerator,
   
   if (enumerator->priv->pending)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_PENDING,
-		   _("File enumerator has outstanding operation"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_PENDING,
+                           _("File enumerator has outstanding operation"));
       return FALSE;
     }
 
@@ -336,8 +390,9 @@ g_file_enumerator_next_files_async (GFileEnumerator     *enumerator,
  * 
  * Finishes the asynchronous operation started with g_file_enumerator_next_files_async().
  * 
- * Returns: a #GList of #GFileInfo<!---->s. You must free the list with g_list_free
- * and unref the infos with g_object_unref when your done with them.
+ * Returns: a #GList of #GFileInfo<!---->s. You must free the list with 
+ *     g_list_free() and unref the infos with g_object_unref when you're 
+ *     done with them.
  **/
 GList *
 g_file_enumerator_next_files_finish (GFileEnumerator  *enumerator,
@@ -525,6 +580,24 @@ g_file_enumerator_set_pending (GFileEnumerator *enumerator,
   enumerator->priv->pending = pending;
 }
 
+/**
+ * g_file_enumerator_get_container:
+ * @enumerator: a #GFileEnumerator
+ *
+ * Get the #GFile container which is being enumerated.
+ *
+ * Returns: the #GFile which is being enumerated.
+ *
+ * Since: 2.18.
+ */
+GFile *
+g_file_enumerator_get_container (GFileEnumerator *enumerator)
+{
+  g_return_val_if_fail (G_IS_FILE_ENUMERATOR (enumerator), NULL);
+
+  return enumerator->priv->container;
+}
+
 typedef struct {
   int                num_files;
   GList             *files;
